
#import <Foundation/Foundation.h>

// Bluetooth Objective C Headers:
#undef MAC_OS_X_VERSION_MIN_REQUIRED
#define MAC_OS_X_VERSION_MIN_REQUIRED MAC_OS_X_VERSION_10_4
#import <IOBluetooth/objc/IOBluetoothSDPServiceRecord.h>
#import <IOBluetooth/objc/IOBluetoothRFCOMMChannel.h>

// Bluetooth C Headers:
#include <IOBluetooth/IOBluetoothUserLib.h>

@class IOBluetoothUserNotification;

@interface BluetoothServerInterface : NSObject {
    // This is the RFCOMM channel we connect to:
    IOBluetoothRFCOMMChannel		*RFCOMMChannel;

    // Service Entry for the service we publish
    BluetoothRFCOMMChannelID		serverChannelID;
    BluetoothSDPServiceRecordHandle	serverHandle;
    
    IOBluetoothUserNotification		*incomingChannelNotification;
    
    // This is the method to call in the UI when new data shows up:
    SEL	handleNewDataSelector;
    id	newDataTarget;
    
    // This is the method to call when the RFCOMM channel disappears.
    SEL	handleRemoteDisconnectionSelector;
    id	disconnectionTarget;
    
    // This is the method to call when the RFCOMM channel disappears.
    SEL handleRemoteConnectionSelector;
    id	connectionTarget;

}

// Connection Method:
// returns TRUE if the connection was successful:
- (BOOL)publishService;

// Stops Providing services
// Removes the published services from the SDP dictionary:
// If a connection is in progress it will not interrupt it.
- (void)stopProvidingServices;

// Disconnection:
// closes the channel:
- (void)disconnectFromClient;

// New RFCOMM channel show up:
// A new RFCOMM channel shows up we have to make sure it is the one we are waiting for:
-(void) newRFCOMMChannelOpened:(IOBluetoothUserNotification *)inNotification channel:(IOBluetoothRFCOMMChannel *)newChannel;

// Returns the name of the device we are connected to
// returns nil if not connection:
//- (NSString *)remoteDeviceName;

// Returns the name of the local bluetooth device
- (NSString *)localDeviceName;

// Send Data method
// returns TRUE if all the data was sent:
- (BOOL)sendData:(void*)buffer length:(UInt32)length;

// IOBluetoothRFCOMMChannel delegate methods
- (void)rfcommChannelData:(IOBluetoothRFCOMMChannel*)rfcommChannel data:(void *)dataPointer length:(size_t)dataLength;
- (void)rfcommChannelClosed:(IOBluetoothRFCOMMChannel*)rfcommChannel;

// Registers selector for incoming data:
// tells to this class to call myTarget and myTargetAction when new data shows up:
- (void)registerForNewData:(id)myTarget action:(SEL)actionMethod;

// Registers selector for disconnection:
// tells to this class to call myTarget and myTargetAction when the channel disconnects:
- (void)registerForTermination:(id)myTarget action:(SEL)actionMethod;

// Registers selector for a successful completed connection:
// tells to this class to call myTarget and myTargetAction when the channel disconnects:
- (void)registerForNewConnection:(id)myTarget action:(SEL)actionMethod;


@end
