//
// Caching.js
// FernsehWidget
// Vom Pascal Pfiffner und Sebastian Krau
//


var cacheBase;
var bgCacheReturnFunction;
var bgCacheShows;
var bgCacheRequest;
var bgCachedShows;
var bgCacheListingDay;
var bgCacheChannel;
var bgCacheChannelArray;
var cacheFurtherURL;
var parseErrorOccurred;
var networkErrorOccurred = false;
var weMayCache = true;
var didActuallyLoad = false;
var lastEncounteredHour;
var saveLastEncounteredHour;


function localUrl(channel, listingday)
{
	var date = new Date();
	listingday = parseInt(listingday);
	
	date.setTime(date.getTime() + ((8 == listingday ? -1 : listingday) * 86400000));
	
	debug("listingday "+listingday+" >> localurl date " + date);
	
	localurl = cacheBase + (date.getMonth() + 1) + "-" + date.getDate() + "-" + channel;
		
	return localurl;
}

function clearCache()
{
	// Clering the cache is a matter of fractions of a second, however, it is nice to give some feedback, so fake it
	document.getElementById("clear_cache_button").style.opacity = 0.4;
	setTimeout('document.getElementById("clear_cache_button").style.opacity = 1;', 500);
	FernsehPlugin.emptyCache();
}

//
// returns whether cached data is available for all channels
//
function cachedDataAvailable(channels, listingday)
{
	var channel;
	for (var i = 0; i < channels.length; i++)
	{
		channel = channels[i];
		var channelshows = FernsehPlugin.cat(localUrl(channel, listingday));
		if ((undefined == channelshows) || ("" == channelshows))
		{
			return false;
		}
	}
	return true;
}

//
// Asynchronous caching
// Note: takes an array of channels
//
function backgroundCachedData(channels, listingday, returnFunc)
{
	// check whether a network error occurred. if yes, the returnfunction deals with it
	if (networkErrorOccurred)
	{
		returnFunc();
		bgCacheChannelArray = undefined;
		stopCacheIndicator();
		return;
	}

	bgCacheReturnFunction = returnFunc;
	parseErrorOccurred = false;
	
	var bgCacheShows = new Object();
	
	var everythingThere = true;
	
	var channel;
	for (var i = 0; i < channels.length; i++)
	{
		channel = channels[i];
		if (undefined == channel) continue;
		var channelshows = loadShows(localUrl(channel, listingday));
		if (undefined == channelshows)
		{
			everythingThere = false;
			break;
		}
		bgCacheShows[channel] = channelshows;
	}
	
	bgCacheChannelArray = channels;
	
	if (everythingThere)
	{	
		returnFunc();
		stopCacheIndicator();
		// only update the badge if caching really happened.
		if (null != bgCacheChannel) updateFilterBadge();
		bgCacheChannel = null;
		return;
	} else {
		// else cache them and then return them
		startCacheIndicator();
		setCacheIndicatorText("Lade " + channel);
		lastEncounteredHour = 0;
		loadInBackground(channel, listingday);
	}
}

//
// Starts asynchronous loading
//
function loadInBackground(channel, listingday)
{
	//lastEncounteredHour = 0;
	if (!weMayCache) return;
	bgCacheChannel = channel;
	bgCacheListingDay = parseInt(listingday);
	networkErrorOccurred = false;
	
	bgCachedShows = new Object();
	// http://programm.tvtoday.de/tv/programm/programm.php?ztag=1&sparte=alle&uhrzeit=00%3A00%3A00&sender=RTL
	//var requesturl = "http://programm.tvtoday.de/tv/programm/programm.php?ztag=" + listingday + "&uhrzeit=Ax00&sparte=alle&sender=" + channel;
	var requesturl = "http://programm.tvtoday.de/tv/programm/programm.php?ztag=" + listingday + "&sparte=alle&uhrzeit=00%3A00%3A00&sender=" + channel;
		
	bgCacheRequest = new XMLHttpRequest;
	bgCacheRequest.onreadystatechange = bgProcessReqChange;
	bgCacheRequest.open("GET", requesturl, true);
	bgCacheRequest.setRequestHeader("Cache-Control", "no-cache");
	bgCacheRequest.send(null);
}

function bgProcessReqChange()
{
	if (parseErrorOccurred) return;
	if (bgCacheRequest.readyState != 4) return;

	if ((!((bgCacheRequest.status >= 200) && (bgCacheRequest.status < 400))) || (bgCacheRequest.status == undefined))
	{
		networkErrorOccurred = true;
		//alert("error in bgProcessReqChange():");
		//alert("status "+bgCacheRequest.status);
		//alert("statustext "+bgCacheRequest.statusText);
		backgroundCachedData(bgCacheChannelArray, bgCacheListingDay, bgCacheReturnFunction);
		stopCacheIndicator();
		// showError();
		// deal with error here
		return;
	}
	networkErrorOccurred = false;
	var newShows = parseShows(bgCacheRequest.responseText, bgCacheListingDay);

	if (undefined == newShows)
	{
		networkErrorOccurred = true;
		alert("parseShows() was unsuccessful, abort caching here");
		backgroundCachedData(bgCacheChannelArray, bgCacheListingDay, bgCacheReturnFunction);
		return;
	}
	
	for (var time in newShows)
	{
		bgCachedShows[time] = newShows[time];			
	}
	
	if ("-" != cacheFurtherURL)
	{
		if (!weMayCache) return;

		// not yet done getting data for this day
		var requesturl = "http://programm.tvtoday.de" + cacheFurtherURL;
		bgCacheRequest = new XMLHttpRequest;
		bgCacheRequest.onreadystatechange = bgProcessReqChange;
		bgCacheRequest.open("GET", requesturl, true);
		bgCacheRequest.setRequestHeader("Cache-Control", "no-cache");
		bgCacheRequest.send(null);
	} else {
		// We have all the data for a complete day
		checkAlarmFilter(bgCachedShows, bgCacheChannel);
		saveShows(bgCachedShows, localUrl(bgCacheChannel, bgCacheListingDay));
		
		// call this function again, it knows what other channels have to be cached and calls the callback if all data has been loaded
		backgroundCachedData(bgCacheChannelArray, bgCacheListingDay, bgCacheReturnFunction);
	}
}

//
// Synchronous caching
// Note: takes the name of one channel
//
function cachedData(channel, listingday)
{
	if (undefined == channel)
	{
		alert("undefined channel!");
		return;
	}
	var shows = loadShows(localUrl(channel, listingday));
	
	if (shows != undefined)
	{	
		return shows;
	} else {
		// else cache them and then return them
		startCacheIndicator();
		setCacheIndicatorText("Lade " + channel);
		
		shows = cacheData(channel, listingday);
		stopCacheIndicator();
		if (undefined == shows)
		{
			// showError();
			return;
		} else {
			updateFilterBadge();
			return shows;
		}
	}
}


//
// Synchronous caching
//
function cacheData(channel, listingday)
{
	var shows = new Object();
	
	saveLastEncounteredHour = lastEncounteredHour;
	lastEncounteredHour = 0;

	if (undefined == listingday) return;
	
	var requesturl = "http://programm.tvtoday.de/tv/programm/programm.php?ztag=" + listingday + "&sparte=alle&uhrzeit=00%3A00%3A00&sender=" + channel;

	cacheFurtherURL = "";
	
	while ("-" != cacheFurtherURL)
	{	
		var request = new XMLHttpRequest;
		
		request.open("GET", requesturl, false);
		request.setRequestHeader("Cache-Control", "no-cache");
		request.send(null);
		
		if (!((request.readyState == 4) && (request.status >= 200) && (request.status < 400))) return;
		
		var newShows = parseShows(request.responseText, listingday);
		
		if (undefined == newShows)
		{
			alert("parseShows() was unsuccessful, abort caching here");
			return;
		}
		
		for (var time in newShows)
		{
			shows[time] = newShows[time];			
		}
		requesturl = "http://programm.tvtoday.de" + cacheFurtherURL;
	}
	
	checkAlarmFilter(shows, channel);
	saveShows(shows, localUrl(channel, listingday));
	lastEncounteredHour = saveLastEncounteredHour;
	return shows;
}

function saveShows(shows, filename)
{
	var saveShows = new Object();
	for (var time in shows)
	{
		var variablesavetime = "_" + time.replace(/\./g, "_");
		saveShows[variablesavetime] = shows[time];
	}
	filename = filename.replace(/ /g, "\\ ");
	var serialized = Serialize(saveShows);
	var encoded = serialized.replace(/"/g, "&quot;");			// " (editor)
	widget.system("/bin/echo \"" + encoded + "\" > " + filename, null);
}

//
// Load from disk
//
function loadShows(filename)
{
	// The plugin handles encoding and spaces in filenames better
	var file = FernsehPlugin.cat(filename);
	
	if ((undefined == file) || ("" == file)) {
		return;
	}
	
	var rawobject = file.replace(/&quot;/g, "\"");
	var deserialized = Deserialize(rawobject);
	var newObj = new Object();
	var foo;
	for(var key in deserialized) {
		foo = key.replace(/_/g, ".");
		newObj[foo.substr(1,5)] = deserialized[key];
	}
	
	return newObj;
}

function parseShows(rawsource, listingday)
{
	// Das alles zu einer Zeile machen 
	var dateFirstShow;
	lines = rawsource.split("\n");
	rsource = lines.join("");
	var newShows = new Object();
	var doBreak = false;
	
	
	// abort if we got no valid data
	if(rawsource.match(/<center><BR><span class="text-farbe">Es tut uns leid,<\/span>  <span class="text">aber wir konnten keine Sendungen mit dieser Suchabfrage finden\.<\/span><\/center>/i)) {
		alert("Ungueltige Suchanfrage an TVToday. Da muss ich passen, sorry.");
		return;
	}
	
	
	/***********************
	 * parsing starts here *
	 ***********************/
	
	try {
		
		// Der entscheidende Teil für das Fernsehprogramm...
		rmatch = /((ContentAd Ende)|(Ende ContentAd)).*?<\/table>.*?<img[^>]+><br>(.*?)<img src="\/tv\/bilder\/linie.gif" width="100%" height="1" border="0"><br><br><table border="0" cellpadding="2" cellspacing="2" width="100%"><tr>(.*)$/;
	
		rmatch.exec(rsource);
		var listing = RegExp.$4;
		
		// relevant later-on for "frhere Sendungen" und "weitere Sendungen"
		var afterlisting = RegExp.$5;
		
		shows = new Array();
		showsregexp = /<table border="0" cellpadding="4" cellspacing="0" width="585">\s?<tr>(.*?)<\/tr>/g;
		rawshows = listing.match(showsregexp);
		
		var startstamp;
		var searchdate;
		var newShows = new Object();
		
		if (8 == listingday) listingday = -1;
		
		// Loop through the single shows
		var lastStartTime;
		
		for (i = 0; i < rawshows.length; i++)
		{
			singleshowregexp = /<td.*?>.*?<\/td>/g;
			thisshow = rawshows[i].match(singleshowregexp);
			
			iconurlregexp = /<td width="38" valign="top"><img src="([^"]*)" [^>]*alt="/;
			iconurlraw = thisshow[0].match(iconurlregexp);
			iconurl = iconurlraw[1];
			
			/*******************************
			 * Start the Date/Time parsing *
			 *******************************/
			
			// RegExp to get the time from tvtoday
			starttimeregexp = /<td valign="top" bgcolor="#a8a8a8"><span class="text-grau">(.*?)<\/span/;
			starttimeraw = thisshow[1].match(starttimeregexp);
			starttime = starttimeraw[1];
						
			// End Date/Time
			
			shownameregexp = /<td valign="middle" width="509"><span class="headline">(<a href.*?><u>)?(.*?)(<\/u>)?<\/span><span class="text">(.*?)\s.*?<\/span>.*?<span class="text">(.*?)\s.*?<\/span>/;
			shownameraw = thisshow[2].match(shownameregexp);
			shownameandchannel = shownameraw[2];
			
			showurlregexp = /<td valign="middle" width="509"><span class="headline">.*?openWindowLupeCustom\('([^']*?)',/;
			showurlraw = thisshow[2].match(showurlregexp);
			var showurl = null;
			if (showurlraw != null) showurl = showurlraw[1];
			
			showchannelregexp = /(.*)\s*\((....*)\)/;
			showchannel = shownameandchannel.match(showchannelregexp);
			
			showname = showchannel[1];
			channel = showchannel[2];
			
			// mit showview
			//showdescregexp = /<td valign="middle" width="476">.*?<span class="text">\s*?<i>(.*?)\s*?<br>\s*?<\/span>.*?<span class="text">\s*?(.*?)\s*?(<br>)?\s*?<\/span>/;
			// ohne showview -- aber wer braucht das?
			showdescregexp = /<td valign="middle" width="509">.*?<span class="text">\s*?<i>(.*?)<\/i>.*?<\/span>.*?<span class="text">\s*?(.*?)\s*?(<br>)?\s*?<\/span>/;
			showdescraw = thisshow[2].match(showdescregexp);
			
			showlength = "";
			showlength = showdescraw[1];
			showdesc = "";
			
			if (undefined != showdescraw[2]) showdesc += showdescraw[2];
			
			var thisHour = Math.abs(starttime.split(".")[0]);
			if (thisHour < lastEncounteredHour)
			{
				doBreak = true;
				break;
			}
			lastEncounteredHour = thisHour;
			
			var thisshow = new Object();
			
			thisshow["title"] = showname;
			thisshow["length"] = showlength;
			if ((undefined != showdesc) && ("" != showdesc)) thisshow["description"] = showdesc;
			if ((undefined != showurl) && ("" != showurl)) thisshow["url"] = showurl;
			if ((undefined != iconurl) && ("" != iconurl)) thisshow["icon"] = iconurl;
			
			// get a real timestamp for the startdate
			thisshowhour = Math.abs((starttime.split("."))[0]);
			
			var date = new Date();
			date.setTime(date.getTime() + (listingday * 86400000));
			date.setHours(thisshowhour);
			date.setMinutes(Math.abs((starttime.split("."))[1]));
			date.setSeconds(0);
			thisshow["timestamp"] = date;
			
			// calculate an endstamp
			thisshow["endtime"] = new Date();
			if("" != showlength) {
				var durparts = showlength.split(" ");
				thisshow["durationInMin"] = Math.abs(durparts[0]);
				thisshow["endtime"].setTime(date.getTime() + (durparts[0] * 60 * 1000));
			}
			else {
				alert("Got no duration for " + thisshow["title"] + " on " + channel + ", assuming 30 Minutes");
				thisshow["durationInMin"] = 30;
				thisshow["endtime"].setTime(Math.abs(date.getTime() + 1800000));		// to be on the safe side and at least have an endstamp (30min after start)
			}
						
			if (!doBreak) newShows[starttime] = thisshow;
		}
		
		// Gibt es weitere Sendungen?
		
		furtherregexp = /<a href="([^"]*)"><span class="link-intern"> weitere Sendungen/; // " for the editor
		furtherraw = afterlisting.match(furtherregexp);
		cacheFurtherURL = "-";
		if ((!doBreak) && (furtherraw != undefined)) cacheFurtherURL = furtherraw[1];
		//if (doBreak) cacheFurtherURL = null;
	}
	
	// error catched
	catch(exc)
	{	
		parseErrorOccurred = true;
		stopCacheIndicator();
		
		document.getElementById('content').style.top = '0px';
	 	document.getElementById("previousShows").style.opacity = 0.4;
		document.getElementById("nextShows").style.opacity = 0.4;
		
		alert("Fehler beim parsen des Programms (Tag " + listingday + ", Source " + rawsource.length + " Zeichen): " + exc);
		showDialog("Beim Analysieren der Daten von TVToday ist ein Fehler aufgetreten.<br /><br />Dies kann unterschiedliche Ursachen haben, der wahrscheinlichste ist da&szlig; TVToday das Format seiner Seiten ver&auml;ndert hat.<br />In diesem Fall k&ouml;nnen Sie nur auf ein Update seitens der Entwickler warten", "Erneut versuchen", updateTVListing, "Webseite &ouml;ffnen", gotoWebsite);
	}
	
	return newShows;
}
